//
//  main.cpp
//  GLSL-Assignment-02
//
//  Created by Orlando Pereira on 08/04/15.
//  Copyright (c) 2015 Orlando Pereira. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>

#include <GLUT/GLUT.h>

#include "textfile.h"

GLuint v,f,f2,p;
float lpos[4] = {1,0.5,1,0};
float a = 0;

void changeSize(int w, int h) {
    
    float ratio = 1.0* w / h;
    
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    
    // Set the viewport to be the entire window
    glViewport(0, 0, w, h);
    
    // Set the correct perspective.
    gluPerspective(45,ratio,1,1000);
    glMatrixMode(GL_MODELVIEW);
}

void renderScene(void) {
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glLoadIdentity();
    gluLookAt(0.0,0.0,5.0, 0.0,0.0,-1.0, 0.0f,1.0f,0.0f);
    
    glLightfv(GL_LIGHT0, GL_POSITION, lpos);
    glRotatef(a,0,1,1);
    
    glBegin(GL_TRIANGLE_STRIP);
    glColor3f(0, 0, 1); glVertex3f(0, 2, 0);
    glColor3f(1, 0, 0); glVertex3f(-1, 0, 1);
    glColor3f(1, 0, 0); glVertex3f(1, 0, 1);
    glColor3f(0, 1, 0); glVertex3f(0, 0, -1.4);
    glColor3f(0, 1, 0); glVertex3f(0, 2, 0);
    glColor3f(0, 0, 1); glVertex3f(-1, 0, 1);
    glEnd();

    
    a+=0.5;
    
    glutSwapBuffers();
}

void processNormalKeys(unsigned char key, int x, int y) {
    if (key == 27)
        exit(0);
}

int printOglError(char *file, int line) {

    GLenum glErr; // Returns 1 if an OpenGL error occurred, 0 otherwise.
    int retCode = 0;
    
    glErr = glGetError();
    while (glErr != GL_NO_ERROR) {
        printf("glError in file %s @ line %d: %s\n", file, line, gluErrorString(glErr));
        retCode = 1;
        glErr = glGetError();
    }
    return retCode;
}

void printShaderInfoLog(GLuint obj) {
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;
    
    glGetShaderiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
    
    if (infologLength > 0) {
        infoLog = (char *)malloc(infologLength);
        glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
        printf("%s\n",infoLog);
        free(infoLog);
    }
}

void printProgramInfoLog(GLuint obj) {
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;
    
    glGetProgramiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
    
    if (infologLength > 0) {
        infoLog = (char *)malloc(infologLength);
        glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
        printf("%s\n",infoLog);
        free(infoLog);
    }
}

void setShaders() {
    
    char *vs = NULL, *fs = NULL;
    
    v = glCreateShader(GL_VERTEX_SHADER);
    f = glCreateShader(GL_FRAGMENT_SHADER);
    f2 = glCreateShader(GL_FRAGMENT_SHADER);
    
    vs = textFileRead("minimal.vert");
    fs = textFileRead("minimal.frag");
    
    const char * vv = vs;
    const char * ff = fs;
    
    glShaderSource(v, 1, &vv,NULL);
    glShaderSource(f, 1, &ff,NULL);
    
    free(vs);free(fs);
    
    glCompileShader(v);
    glCompileShader(f);
    
    printShaderInfoLog(v);
    printShaderInfoLog(f);
    printShaderInfoLog(f2);
    
    p = glCreateProgram();
    glAttachShader(p,v);
    glAttachShader(p,f);
    
    glLinkProgram(p);
    printProgramInfoLog(p);
    
    glUseProgram(p);
}

int main(int argc, char **argv) {
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowPosition(100,100);
    glutInitWindowSize(320,320);
    glutCreateWindow("Hello GLSL");
    
    glutDisplayFunc(renderScene);
    glutIdleFunc(renderScene);
    glutReshapeFunc(changeSize);
    glutKeyboardFunc(processNormalKeys);
    
    glEnable(GL_DEPTH_TEST);
    glClearColor(1.0,1.0,1.0,1.0);
    glEnable(GL_CULL_FACE);
    
    setShaders();
    
    glutMainLoop();
    
    return 0;
}

